set -e # Exit shell when an error is encountered

if [ "$1" = "-h" ]; then
  echo "Usage: `basename $0` [DB_NAME (default=ram)]"
  exit 0
fi

# Config
DB_NAME=${1:-"ram"}
echo "DB_NAME=$DB_NAME"

# Check project
CURRENT_PROJECT=$(oc project --short)
echo "CURRENT_PROJECT=$CURRENT_PROJECT"
if [ "${CURRENT_PROJECT:0:7}" != "ns-res-" ]; then
  echo "Forbidden: This is not a Resin project"
  exit 0
fi
if [ "${CURRENT_PROJECT:0:8}" == "ns-res-p" ]; then
  echo "Forbidden: This is the production project"
  exit 0
fi

# Restore of the source backup
curl -kLSs ${BACKUP_URL}/latest.tar.gz -o latest.tar.gz
curl -kLSs ${BACKUP_URL}/.migrate -o .migrate
tar -xvf ./latest.tar.gz
oc exec deploy/res-db-deployment -- bash -c 'rm -rf /tmp/dump'
oc exec deploy/res-db-deployment -- bash -c 'mkdir /tmp/dump'
DB_POD_NAME=$(oc get pods -l app=res-db -o jsonpath='{.items[0].metadata.name}')
echo "DB_POD_NAME=$DB_POD_NAME"
oc cp ./mongo__*/ram/ $DB_POD_NAME:/tmp/dump/$DB_NAME
oc exec deploy/res-db-deployment -- bash -c "mongo --authenticationDatabase admin --username root --password \$MONGODB_ROOT_PASSWORD $DB_NAME --eval 'db.dropDatabase()'"
oc exec deploy/res-db-deployment -- bash -c "mongorestore --authenticationDatabase admin --username root --password \$MONGODB_ROOT_PASSWORD -d $DB_NAME /tmp/dump/$DB_NAME/"
echo "***** Latest backup restored in $DB_POD_NAME in db $DB_NAME"

# Cleanup user email and password change (not for admin users)
oc cp $(dirname "$(realpath "$0")")/mongo_clean_users.js "${DB_POD_NAME}:/tmp/dump/"
oc exec deploy/res-db-deployment -- bash -c "mongo --authenticationDatabase admin --username root --password \$MONGODB_ROOT_PASSWORD $DB_NAME < /tmp/dump/mongo_clean_users.js"

# .migrate file copy
if [ "$DB_NAME" = "ram" ]; then 
  oc exec deploy/res-maintenance-deployment -- bash -c 'mv -f /pvc-files/.migrate /tmp/.migrate'
  oc cp ./.migrate $(oc get pods -l app=res-maintenance -o jsonpath='{.items[0].metadata.name}'):/pvc-files/.migrate
fi
oc exec deploy/res-db-deployment -- bash -c 'rm -rf /tmp/dump/'

echo ""
echo "*********************************************************** Summary ***********************************************************"
echo "Latest backup restored in $DB_POD_NAME in db $DB_NAME (and data cleaned up)"
if [ "$DB_NAME" = "ram" ]; then echo "The local .migrate file has been copied"; fi
if [ "$DB_NAME" = "ram" ]; then echo "Don't forget to reset elasticsearch indexes!!!"; fi
echo "*******************************************************************************************************************************"

exit 0
