import { HttpModule } from '@nestjs/axios';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { CreateCategories } from '../dto/create-categories.dto';
import { CategoriesService } from './categories.service';

describe('CategoriesService', () => {
  let categoriesService: CategoriesService;

  const mockCategoriesModel = {
    create: jest.fn(),
    find: jest.fn(),
    findOne: jest.fn(),
    select: jest.fn(),
    exec: jest.fn(),
  };

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [HttpModule],
      providers: [
        CategoriesService,
        {
          provide: getModelToken('Categories'),
          useValue: mockCategoriesModel,
        },
      ],
    }).compile();

    categoriesService = module.get<CategoriesService>(CategoriesService);
  });

  it('should be defined', () => {
    expect(categoriesService).toBeDefined();
  });

  it('should findAll structures', async () => {
    const data: CreateCategories[] = [
      { id: 'categoryId', theme: 'Public', name: 'test', modules: [{ id: 'Test', name: 'Text du test' }] },
      { id: 'categoryId', theme: 'Public', name: 'test2', modules: [{ id: 'Test2', name: 'Text du test test 2' }] },
    ];
    mockCategoriesModel.find.mockReturnThis();
    mockCategoriesModel.exec.mockResolvedValueOnce(data);
    expect(await categoriesService.findAll()).toEqual(data);
  });
  it('should findOneComplete structures', async () => {
    const data: CreateCategories[] = [
      { id: 'categoryId', theme: 'Public', name: 'test', modules: [{ id: 'Test', name: 'Text du test' }] },
      { id: 'categoryId', theme: 'Public', name: 'test2', modules: [{ id: 'Test2', name: 'Text du test test 2' }] },
    ];
    mockCategoriesModel.findOne.mockReturnThis();
    mockCategoriesModel.exec.mockResolvedValueOnce(data);
    expect(await categoriesService.findOneComplete('categoryId')).toEqual(data);
  });
});
