import { HttpService } from '@nestjs/axios';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import {
  espaceCoopCNFSMockData,
  espaceCoopPermanencesMockData,
} from '../../../test/mock/data/espaceCoopCNFS.mock.data';
import { usersMockData } from '../../../test/mock/data/users.mock.data';
import { UsersService } from '../../users/services/users.service';
import { EspaceCoopService } from './espaceCoop.service';

describe('EspaceCoopCNFSService', () => {
  let espaceCoopService: EspaceCoopService;

  const mockEspaceCoopCNFSModel = {
    create: jest.fn(),
    find: jest.fn(() => mockEspaceCoopCNFSModel),
  };

  const httpServiceMock = {
    get: jest.fn(),
  };

  const usersServiceMock = {
    findOne: jest.fn(),
  };

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      providers: [
        EspaceCoopService,
        {
          provide: getModelToken('EspaceCoopCNFS'),
          useValue: mockEspaceCoopCNFSModel,
        },
        {
          provide: HttpService,
          useValue: httpServiceMock,
        },
        {
          provide: UsersService,
          useValue: usersServiceMock,
        },
      ],
    }).compile();

    espaceCoopService = module.get<EspaceCoopService>(EspaceCoopService);
  });

  it('should be defined', () => {
    expect(espaceCoopService).toBeDefined();
  });

  describe('parsePermanences', () => {
    it('should parse correctly permanences', async () => {
      // Mock finding first cnfs in user database
      jest.spyOn(usersServiceMock, 'findOne').mockImplementationOnce(() => usersMockData[0]);
      const result = await espaceCoopService.parsePermanences(espaceCoopPermanencesMockData);
      expect(result).toStrictEqual(espaceCoopCNFSMockData);
    });
  });
});
