import { Body, Controller, Get, Logger, Post, Query } from '@nestjs/common';
import { ApiTags } from '@nestjs/swagger';
import { OrientationIndicator } from '../schemas/orientation-indicator.schema';
import { IndicatorService } from '../services/indicator.service';
import { IndicatorQueryDto } from '../dto/indicator-query.dto';

@ApiTags('indicator')
@Controller('indicator')
export class IndicatorController {
  private readonly logger = new Logger(IndicatorController.name);
  constructor(private readonly indicatorService: IndicatorService) {}

  @Post('orientation')
  public async createOrientation(@Body() orientationIndicator: OrientationIndicator): Promise<OrientationIndicator> {
    return this.indicatorService.createOrientation(orientationIndicator);
  }

  @Get('orientation')
  public async getAllOrientation(@Query() queryDto: IndicatorQueryDto): Promise<OrientationIndicator[]> {
    return this.indicatorService.getAllOrientation(queryDto.limit, queryDto.skip, queryDto.startDate, queryDto.endDate);
  }

  @Get('users')
  public async countUsers(@Query() queryDto: IndicatorQueryDto): Promise<number> {
    return this.indicatorService.countUsers(queryDto.startDate, queryDto.endDate);
  }
}
