import { Type } from 'class-transformer';
import { IsBoolean, IsNumber, IsOptional, IsString, ValidateNested } from 'class-validator';
import { Address } from '../../structures/schemas/address.schema';

class StructureDto {
  @IsString()
  nom: string;

  @ValidateNested()
  @Type(() => Address)
  adresse: Address;

  @IsOptional() // This makes the next validations optional, remove it if zone CTM and distance are always required.
  @IsString()
  zoneCTM?: string;

  @IsOptional() // If distance is always required, you can remove this.
  @IsNumber()
  distance?: number;
}

class TypeDto {
  @IsString()
  typeOrientation: 'accès matériel' | 'achat matériel' | 'démarche en ligne' | 'compétences';

  @IsString({ each: true })
  filtres: string[];

  @IsString()
  typeAccompagnement: 'rdv en ligne' | 'médiation' | 'trouver une structure';
}

export class OrientationIndicator {
  @ValidateNested()
  @Type(() => StructureDto)
  origin: {
    nom: string;
    prescripteur: string;
    adresse: Address;
  };

  @ValidateNested({ each: true })
  @Type(() => StructureDto)
  target: StructureDto[];

  @ValidateNested()
  @Type(() => TypeDto)
  type: TypeDto;

  @IsBoolean()
  connected: boolean;

  @IsBoolean()
  completed: boolean;

  @IsNumber()
  progress: number;

  createdAt: Date;
}
