import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { mockIndicator, mockIndicators } from '../../../test/mock/data/indicators/orientationIndicators.mock.data';
import { User } from '../../users/schemas/user.schema';
import { OrientationIndicator } from '../schemas/orientation-indicator.schema';
import { IndicatorService } from './indicator.service';
import { StructuresService } from '../../structures/services/structures.service';
import { StructuresServiceMock } from '../../../test/mock/services/structures.mock.service';

describe('IndicatorService', () => {
  let indicatorService: IndicatorService;
  let mockModel;

  beforeEach(async () => {
    mockModel = {
      find: jest.fn(),
      create: jest.fn(),
    };

    const module = await Test.createTestingModule({
      providers: [
        IndicatorService,
        {
          provide: getModelToken(OrientationIndicator.name),
          useValue: mockModel,
        },
        {
          provide: getModelToken(User.name),
          useValue: User,
        },
        {
          provide: StructuresService,
          useClass: StructuresServiceMock,
        },
      ],
    }).compile();

    indicatorService = module.get<IndicatorService>(IndicatorService);
  });

  it('should be defined', () => {
    expect(indicatorService).toBeDefined();
  });

  describe('createOrientation', () => {
    it('should create orientation indicator', async () => {
      mockModel.create.mockResolvedValue(mockIndicator);

      const result = await indicatorService.createOrientation(mockIndicator);
      expect(result).toBe(mockIndicator);
      expect(mockModel.create).toHaveBeenCalledWith(mockIndicator);
    });

    it('should handle errors when creating orientation indicator', async () => {
      mockModel.create.mockRejectedValue(new Error('Some error'));

      await expect(indicatorService.createOrientation(mockIndicator)).rejects.toThrow('Some error');
    });
  });

  describe('getAllOrientation', () => {
    it('should retrieve all orientation indicators', async () => {
      mockModel.find.mockResolvedValue(mockIndicators);

      const result = await indicatorService.getAllOrientation();
      expect(result).toBe(mockIndicators);
      expect(mockModel.find).toHaveBeenCalledWith({});
    });

    it('should handle errors when fetching all orientation indicators', async () => {
      mockModel.find.mockRejectedValue(new Error('Some fetching error'));

      await expect(indicatorService.getAllOrientation()).rejects.toThrow('Some fetching error');
    });
  });
});
