import { LogLevel, ValidationPipe } from '@nestjs/common';
import { NestFactory } from '@nestjs/core';
import { DocumentBuilder, SwaggerModule } from '@nestjs/swagger';
import { AppModule } from './app.module';

async function bootstrap() {
  const logLevelInfo: LogLevel[] = ['error', 'warn', 'log'];
  const logLevelDebug: LogLevel[] = ['error', 'warn', 'log', 'debug', 'verbose'];
  let logLevel: LogLevel[];

  // Use NODE_LOG if exists, else use NODE_ENV
  switch (process.env.NODE_LOG) {
    case 'log':
      logLevel = logLevelInfo;
      break;
    case 'debug':
      logLevel = logLevelDebug;
      break;
    default:
      logLevel = process.env.NODE_ENV === 'production' ? logLevelInfo : logLevelDebug;
      break;
  }

  const app = await NestFactory.create(AppModule, {
    logger: logLevel,
  });
  app.useGlobalPipes(new ValidationPipe());
  const options = new DocumentBuilder()
    .setTitle(`Res'in`)
    .setDescription(`Res'in API documentation`)
    .setVersion('1.0')
    .addBearerAuth({ type: 'http', scheme: 'bearer', bearerFormat: 'JWT' }, 'JWT')
    .build();
  app.setGlobalPrefix('api');
  const document = SwaggerModule.createDocument(app, options);
  SwaggerModule.setup('api/doc', app, document);
  await app.listen(3000);
}
bootstrap();
