import { Db, ObjectId } from 'mongodb';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();
  const documents = [
    { category: 'Publique', value: 'laPoste', selectable: 'true' },
    { category: 'Publique', value: 'espaceEmploi', selectable: 'true' },
    { category: 'Publique', value: 'CPAM', selectable: 'true' },
  ];
  try {
    await db.collection('structuretype').insertMany(documents);
    await db.collection('structuretype').deleteOne({ value: 'pimms' });
    console.log('Updated : laPoste, espaceEmploi and CPAM added to "StructureType" collection and pimms removed');
  } catch (error) {
    console.error('Error updating documents:', error);
  }
};

export const down = async () => {
  const db: Db = await getDb();
  const filter = {
    $or: [{ value: 'laPoste' }, { value: 'espaceEmploi' }, { value: 'CPAM' }],
  };
  try {
    await db.collection('structuretype').deleteMany(filter);
    await db.collection('structuretype').insertOne({
      _id: new ObjectId('63bd3c362255d6776457e5c2'),
      category: 'Privée à but non lucratif',
      value: 'pimms',
      selectable: 'true',
    });
    console.log('Downgraded : laPoste, espaceEmploi and CPAM removed from "StructureType" collection and pimms added');
  } catch (error) {
    console.error('Error downgrading documents:', error);
  }
};
