import { Db } from 'mongodb';
import { sanitize } from '../../shared/utils';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();

  const usersCursor = db.collection('users').find({});
  let document;
  while ((document = await usersCursor.next())) {
    let permalink = `${sanitize(document.name)}-${sanitize(document.surname)}`;

    // If already exists, add a number to the end of the permalink
    const exists = await db.collection('users').findOne({ permalink });
    if (exists) {
      let count = 1;
      while (await db.collection('users').findOne({ permalink: `${permalink}-${count}` })) {
        count++;
      }
      permalink = `${permalink}-${count}`;
    }
    await db.collection('users').updateOne({ _id: document._id }, [{ $set: { permalink } }]);
  }

  const structuresCursor = db.collection('structures').find({});
  while ((document = await structuresCursor.next())) {
    let permalink = sanitize(document.structureName);

    // If already exists, add a number to the end of the permalink
    const exists = await db.collection('structures').findOne({ permalink });
    if (exists) {
      let count = 1;
      while (await db.collection('structures').findOne({ permalink: `${permalink}-${count}` })) {
        count++;
      }
      permalink = `${permalink}-${count}`;
    }
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: { permalink } }]);
  }
  console.log(`Update done : added permalink to users and structures`);
};

export const down = async () => {
  const db: Db = await getDb();

  await db.collection('users').updateMany({}, [{ $unset: 'permalink' }]);
  await db.collection('structures').updateMany({}, [{ $unset: 'permalink' }]);

  console.log(`Downgrade done : removed permalink from users and structures`);
};
