import { Db } from 'mongodb';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  try {
    const db: Db = await getDb();
    const structures = await db.collection('structures').find().toArray();

    for (const structure of structures) {
      let freeWorkShopValue: string;
      if (typeof structure.freeWorkShop === 'boolean') {
        freeWorkShopValue = structure.freeWorkShop ? 'Oui' : 'Non';
      } else if (structure.freeWorkShop === 'underCondition') {
        freeWorkShopValue = 'Oui, sous condition';
      } else {
        freeWorkShopValue = structure.freeWorkShop;
      }

      await db
        .collection('structures')
        .updateOne(
          { _id: structure._id },
          { $set: { 'categories.freeWorkShop': freeWorkShopValue }, $unset: { freeWorkShop: '' } }
        );
    }

    console.log(`Update done: Fixed invalid freeWorkShops values and move it to categories.`);
  } catch (error) {
    console.error('Error while fixing moving freeWorkShops to categories:', error);
  }
};

export const down = async () => {
  try {
    const db: Db = await getDb();
    const structures = await db.collection('structures').find().toArray();

    for (const structure of structures) {
      const { categories } = structure;

      await db
        .collection('structures')
        .updateOne(
          { _id: structure._id },
          { $set: { freeWorkShop: categories.freeWorkShop }, $unset: { 'categories.freeWorkShop': '' } }
        );
    }

    console.log(`Downgrade done: moved freeWorkShop back from categories`);
  } catch (error) {
    console.error('Error moving back freeWorkShop:', error);
  }
};
