import { Db } from 'mongodb';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();
  const usersCollection = db.collection('users');

  const usersWithPendingLinks = await usersCollection
    .find({
      pendingStructuresLink: { $exists: true, $ne: [] },
    })
    .toArray();

  for (const user of usersWithPendingLinks) {
    const updatedLinks = user.pendingStructuresLink.map((link) => {
      if (typeof link.createdAt === 'string') {
        link.createdAt = new Date(link.createdAt);
      }
      return link;
    });

    await usersCollection.updateOne({ _id: user._id }, { $set: { pendingStructuresLink: updatedLinks } });
  }

  console.log('Update done: pending structures links createdAt field converted to ISODate');
};

export const down = async () => {
  const db: Db = await getDb();
  const usersCollection = db.collection('users');

  const usersWithPendingLinks = await usersCollection
    .find({
      pendingStructuresLink: { $exists: true, $ne: [] },
    })
    .toArray();

  for (const user of usersWithPendingLinks) {
    const revertedLinks = user.pendingStructuresLink.map((link) => {
      if (link.createdAt instanceof Date) {
        link.createdAt = link.createdAt.toISOString();
      }
      return link;
    });

    await usersCollection.updateOne({ _id: user._id }, { $set: { pendingStructuresLink: revertedLinks } });
  }

  console.log('Downgrade done: pending structures links createdAt field converted to string');
};
