import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { Types } from 'mongoose';
import { PersonalOfferDocument } from '../../personal-offers/schemas/personal-offer.schema';
import { UserRole } from '../enum/user-role.enum';
import { pendingStructuresLink } from '../interfaces/pendingStructure';
import { Employer } from './employer.schema';
import { JobDocument } from './job.schema';

@Schema({ timestamps: true })
export class User {
  @Prop()
  createdAt: Date;

  @Prop({ required: true })
  email: string;

  @Prop({ required: true })
  name: string;

  @Prop({ required: true })
  surname: string;

  @Prop({ required: true })
  phone: string;

  @Prop({ required: true })
  password: string;

  @Prop({ default: false })
  emailVerified: boolean;

  @Prop({ default: null })
  validationToken: string;

  @Prop({ default: null })
  resetPasswordToken: string;

  @Prop({ enum: [UserRole.admin, UserRole.user], default: UserRole.user })
  role: number;

  @Prop({ default: null })
  changeEmailToken: string;

  @Prop({ default: null })
  newEmail: string;

  @Prop({ default: null })
  structuresLink: Types.ObjectId[];

  @Prop({ default: null })
  pendingStructuresLink: pendingStructuresLink[];

  @Prop({ default: null })
  structureOutdatedMailSent: Types.ObjectId[];

  @Prop({ default: null })
  unattachedSince: Date;

  @Prop({ type: [{ type: Types.ObjectId, ref: 'PersonalOffer' }] })
  personalOffers: PersonalOfferDocument[];

  @Prop({ type: Types.ObjectId, ref: 'Employer' })
  employer?: Employer;

  @Prop({ type: Types.ObjectId, ref: 'Job' })
  job?: JobDocument;

  @Prop({ required: false })
  withAppointment?: boolean;

  @Prop({ required: false })
  description?: string;

  @Prop({ default: null })
  lastLoginDate: Date;

  @Prop()
  permalink: string;

  // Document methods
  // static because object method would need to create a constructor and get an actual User instance (cf. https://stackoverflow.com/a/42899705 )

  /** Get if owner can have appointment  */
  static getWithAppointment(user: User) {
    if (user.job && !user.job.name) {
      throw new Error(`job not populated for user ${user.surname} : ${user.job}`);
    }
    // Also check user has a job with personalOffer (user may have changed his job after choosing withAppointment)
    // and also check if user has personalOffers (otherwise they can't have appointment)
    return user.withAppointment && user.job?.hasPersonalOffer && user.personalOffers?.length;
  }
}

export const UserSchema = SchemaFactory.createForClass(User);
UserSchema.index({ name: 'text', surname: 'text' });
