import { ElasticsearchService } from '@nestjs/elasticsearch';
import { Test } from '@nestjs/testing';
import { multipleUsers } from '../../../test/mock/data/users.mock.data';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { SearchModule } from '../../search/search.module';
import { IUserRegistry } from '../interfaces/userRegistry.interface';
import { UserRegistrySearchService } from './userRegistry-search.service';

describe('UserRegistrySearchService Search cases', () => {
  let userRegistrySearchService: UserRegistrySearchService;

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [SearchModule, ConfigurationModule],
      providers: [UserRegistrySearchService],
    }).compile();

    userRegistrySearchService = module.get<UserRegistrySearchService>(UserRegistrySearchService);
    userRegistrySearchService['index'] = 'user-unit-test';
    // Init test cases
    await userRegistrySearchService.dropIndex();
    await userRegistrySearchService.createUserRegistryIndex();
    await Promise.all(multipleUsers.map((user) => userRegistrySearchService.indexUserRegistry(user)));

    // wait for the new structures to be indexed before search
    await userRegistrySearchService.refreshIndexUserRegistry();
    await new Promise((r) => setTimeout(r, 2000));
  }, 10000);

  it('should be defined', async () => {
    expect(userRegistrySearchService).toBeDefined();
  });
  describe('Search method', () => {
    it('should find Guilhem', async () => {
      const res = await userRegistrySearchService.search('Guilhem');
      expect(res[0].surname).toBe('Guilhem');
      expect(res.length).toBe(1);
    });
    it('should find adm', async () => {
      const res = await userRegistrySearchService.search('adm');
      expect(res[0].name).toBe('Admin');
      expect(res.length).toBe(1);
    });
    it('should find empty string', async () => {
      const res = await userRegistrySearchService.search('');
      expect(res.length).toBe(6);
    });
  });
  describe('Indexation methods', () => {
    it('should index User', async () => {
      const res = await Promise.all(multipleUsers.map((user) => userRegistrySearchService.indexUserRegistry(user)));
      expect(res).toBeTruthy();
      expect(res.length).toBe(6);
    });
    it('should update index', async () => {
      const res = await userRegistrySearchService.update(multipleUsers[0] as IUserRegistry);
      expect(res).toBeTruthy();
    });
    it('should delete index', async () => {
      const mockDelete = jest.fn();
      jest.spyOn(ElasticsearchService.prototype, 'delete').mockImplementation(mockDelete);
      const resAdm = await userRegistrySearchService.search('adm');
      await userRegistrySearchService.deleteIndex(resAdm[0] as IUserRegistry);
      expect(mockDelete).toHaveBeenCalled();
    });
  });
});
